#!/usr/bin/perl
# Author: jmarch
# Email : jmarch@hanmail.net
# License: You can freely modify and redistribute

###################################################################################
# WARNING: The author does not warrant anything that caused by using this program #
###################################################################################

use strict;
use DBI;
use File::Find;
use Encode;

$| = 1;

# input basic information
print "host name (default = localhost): ";
my $hostname = <STDIN>;
chomp $hostname;
$hostname ||= 'localhost';
print "database name (default = $ENV{USER}): ";
my $database = <STDIN>;
chomp $database;
$database ||= $ENV{USER};
print "DB user name (defaul = $ENV{USER}): ";
my $username = <STDIN>;
chomp $username;
$username ||= $ENV{USER};
print "DB password: ";
system 'stty -echo';
my $password = <STDIN>;
chomp $password;
system 'stty echo';

# DB handler
my $dbh = DBI->connect( "DBI:mysql:host=$hostname;database=$database", $username, $password );
if (!$dbh) {
    print "ERROR: Failed to connect DB\n";
    exit;
}

# show DB tables, input prefix
my @tables;
my $sth = $dbh->prepare( 'SHOW TABLES' );
$sth->execute();
print "Current DB tables:\n";
while ( my @val = $sth->fetchrow_array() ) {
    print "  $_\n" for (@val);
    push @tables, $val[0];
}
print "\n";
print "Prefix for zeroboard4 (default = zetyx): ";
my $prefix = <STDIN>;
chomp $prefix;
$prefix ||= 'zetyx';

# input board_table
my @zb_tables;
my $sth = $dbh->prepare( "SELECT name FROM ${prefix}_admin_table ORDER BY no" );
$sth->execute;
my $num = 1;
while ( my $tb = $sth->fetchrow_hashref ) {
    print "  $num : $tb->{name}\n"; 
    push @zb_tables, $tb->{name};
    $num++;
}
print "If nothing is listed above, the prefix, $prefix might be wrong.\n";
print "List the number of the table to be converted to a BLOG (separate numbers with spaces): ";
my $ans = <STDIN>;
chomp $ans;
my @index = split /\s+/, $ans;
print "You chose ". join (', ', @index);
print "\n";
my @board;
if ( scalar @index ) {
    for ( @index ) {
        $_--;
    }
    @board = @zb_tables[@index];
}

print "List the number of the table to be converted to a GUESTBOOK (separate numbers with spaces): ";
my $ans = <STDIN>;
chomp $ans;
my @index = split /\s+/, $ans;
print "You chose ". join (', ', @index);
print "\n";
my @gb;
if ( scalar @index ) {
    for ( @index ) {
        $_--;
    }
    @gb = @zb_tables[@index];
}

my %attach_map = create_attach_map( @board );
open XML, '>', "zb4_tt11.xml";
print XML qq{<?xml version="1.0" encoding="utf-8" ?>\n};
print XML qq{<blog type="tattertools/1.1" migrational="false">\n};
write_board_xml( @board, \%attach_map );
write_gb_xml( @gb );
print XML qq{</blog>\n};
close XML; 
print "Done!\n";

# disconnect DB
$dbh->disconnect();

# subroutines
sub create_attach_map {
    my @board = @_;

    mkdir 'attach';
    mkdir 'attach/1';
    my %map;
    for my $bd ( @board ) {
        my $serial = 1;
        find( sub {
            next if ( -d $_ );
            /(\.[^\.]*)$/;
            my $ext = $1;
# ȣ ִ  ó; add by chj
	    my $filename = $File::Find::name ; 
	    $filename =~ s/\(/\\(/g ;
	    $filename =~ s/\)/\\)/g ;
# space ִ  ó; add by chj
	    $filename =~ s/\ /\\ /g ;
            $map{$filename} = "zb4_${bd}_$serial".$ext;
            $serial++;
        }, "data/$bd" );
    }
    for my $key ( %map ) {
        system "cp $key attach/1/$map{$key}" if ( ! -e "attach/1/$map{$key}");
    }
    return %map;
}

sub write_board_xml {
    my $map   = pop;
    my @board = @_;

    # admin_table
    my %property;
    for my $bd ( @board ) {
        my $sth = $dbh->prepare( qq{SELECT * FROM ${prefix}_admin_table WHERE name = "$bd"} );
        $sth->execute;
        $property{$bd} = $sth->fetchrow_hashref;
    }

    # category
    my %category;
    for my $bd ( @board ) {
        my $sth = $dbh->prepare( "SELECT * FROM ${prefix}_board_category_$bd" );
        $sth->execute;
        while ( my $cat = $sth->fetchrow_hashref ) {
            $cat->{no} = xml_encode( $cat->{no} );
            $category{$bd}{ $cat->{no} } = encode( "utf-8", decode( "euc-kr", $cat->{name} ) );
        }
    }
    my $priority = 1;
    for my $bd ( keys %category ) {
        print XML qq{<category>\n};
        print XML qq{  <name>$bd</name>\n};
        print XML qq{  <priority>$priority</priority>\n};
        $priority++;
        for my $cat ( keys %{ $category{$bd} } ) {
            print XML qq{  <category>\n};
            print XML qq{    <name>$category{$bd}{$cat}</name>\n};
            print XML qq{    <priority>$priority</priority>\n};
            $priority++;
            print XML qq{  </category>\n};
        }
        print XML qq{</category>\n};
    }
    
    my $id = 1;
    my %sth;
    for my $bd ( @board ) {
        $sth{$bd} = $dbh->prepare( "SELECT * FROM ${prefix}_board_$bd ORDER BY reg_date" );
        $sth{$bd}->execute;
    }

    my %row;
    for ( @board ) { $row{$_} = '' };
    while (1) {
        for my $bd ( keys %row ) {
            if ( !$row{$bd} ) {
                $row{$bd} = $sth{$bd}->fetchrow_hashref();
                delete $row{$bd} if ( ! $row{$bd} );
            }
        }
        my $tag;
        my %reg_date;
        for my $bd ( keys %row ) {
            if ( $row{$bd} ) {
                $tag++;
                $reg_date{$bd} = $row{$bd}{reg_date};
            }
        }
        last if ( ! $tag );

        my $bd = (sort { $reg_date{$a} <=> $reg_date{$b} } keys %reg_date)[0];
        my $href = $row{$bd};
            
        $href->{memo} =~ s{\n}{<br />}g;
        # board encoding (euc-kr to utf-8)
        for my $key ( keys %{$href} ) {
            if ( $key ne 'file_name1' && $key ne 'file_name2' ) {
                 $href->{$key} = xml_encode( $href->{$key} );
            }
            $href->{$key} = encode( "utf-8", decode( "euc-kr", $href->{$key} ) );
        }
        my %data;
        $data{id}              = $id++;
        $data{visibility}      = $href->{is_secret} ? 'private' : 'public';
        $data{title}           = $href->{subject};
        $data{content}         = $href->{memo};
        $data{location}        = '';
        $data{password}        = $href->{password};
        $data{acceptComment}   = $property{$bd}{use_comment};
        $data{acceptTrackback} = 1;
        $data{published}       = $href->{reg_date};
        $data{created}         = $href->{reg_date};
        $data{modified}        = $href->{reg_date};
        $data{category}        = $bd . '/' . $category{$bd}{ $href->{category} };
	$data{use_html}        = $href->{use_html};

        my @attachment;
        $attachment[0]{mime}  = mime_type( $href->{file_name1} );
        $attachment[0]{size}  = (stat encode("euc-kr",decode("utf-8",$href->{file_name1})))[7];
# ѱϸ ó: edited by chj
        $attachment[0]{name}  = $map->{ encode("euc-kr",decode("utf-8",$href->{file_name1})) };
        $attachment[0]{label} = $href->{s_file_name1};
        $attachment[0]{enclosure} = 0;
        $attachment[0]{attached}  = $href->{reg_date};
        $attachment[0]{downloads} = $href->{download1};
        $attachment[1]{mime}  = mime_type( $href->{file_name2} );
        $attachment[1]{size}  = (stat encode("euc-kr",decode("utf-8",$href->{file_name2})))[7];
# ѱϸ ó: edited by chj
        $attachment[1]{name}  = $map->{ encode("euc-kr",decode("utf-8",$href->{file_name2})) };
        $attachment[1]{label} = $href->{s_file_name2};
        $attachment[1]{enclosure} = 0;
        $attachment[1]{attached}  = $href->{reg_date};
        $attachment[1]{downloads} = $href->{download2};

        if ( $attachment[1]{mime} ) {
	    if ( $data{use_html} ) {
                $data{content} = '&lt;img src=&quot;[##_ATTACH_PATH_##]/' . $attachment[1]{name} . '&quot; alt=&quot;User-created image&quot; style=&quot;float: left;&quot; title=&quot;&quot; /&gt;' . $data{content};
	    }
	    else {
                $data{content} = qq{[##_1C|$attachment[1]{name}|alt=&quot;User-created&quot;|_##]} . $data{content};
	    }
        }
        if ( $attachment[0]{mime} ) {
	    if ( $data{use_html} ) {
                $data{content} = '&lt;img src=&quot;[##_ATTACH_PATH_##]/' . $attachment[0]{name} . '&quot; alt=&quot;User-created image&quot; style=&quot;float: left;&quot; title=&quot;&quot; /&gt;' . $data{content};
	    }
	    else {
                $data{content} = qq{[##_1C|$attachment[0]{name}|alt=&quot;User-created&quot;|_##]} . $data{content};
	    }
        }

        my @comment;
        my $sth2 = $dbh->prepare( qq{SELECT * FROM ${prefix}_board_comment_$bd WHERE parent = "$href->{no}" ORDER BY no} );
        $sth2->execute;
        while ( my $cmt = $sth2->fetchrow_hashref ) {
            # comment encoding (euc-kr to utf-8)
            for my $key ( keys %{$cmt} ) {
                $cmt->{$key} = xml_encode( $cmt->{$key} );
                $cmt->{$key} = encode( "utf-8", decode( "euc-kr", $cmt->{$key} ) );
            }
            my $hcmt;
            $hcmt->{commenter}{name}     = $cmt->{name};
            $hcmt->{commenter}{homepage} = '';
            $hcmt->{commenter}{ip}       = $cmt->{ip};
            $hcmt->{password}            = $cmt->{password};
            $hcmt->{content}             = $cmt->{memo};
            $hcmt->{secret}              = 0;
            $hcmt->{written}             = $cmt->{reg_date};
            $hcmt->{isFiltered}          = 0;
            push @comment, $hcmt;
        }

        $row{$bd} = '';

        print XML qq{<post slogan="$data{title}" format="1.1">\n};
        for my $tag ( qw/ id visibility title content location password acceptComment
                          acceptTrackback published created modified category /
                    ) {
	    if ( $tag eq 'content' ) {
	        if ( $data{use_html} ) {
	            print XML qq{  <content formatter="html" editor="plain">$data{$tag}</content>\n};
	        }
    	        else {
	            print XML qq{  <content>$data{$tag}</content>\n};
                }	        
	    }
	    else {
                print XML "  <$tag>$data{$tag}</$tag>\n";
	    }
        }
        for my $n ( qw/ 0 1 / ) {
            next if ( !$attachment[$n]{label} );
            if ( $attachment[$n]{mime} ) {
                print XML qq{  <attachment mime="$attachment[$n]{mime}" size="$attachment[$n]{size}">\n};
            }
            else {
                print XML qq{  <attachment size="$attachment[$n]{size}">\n};
            }
            for my $tag ( qw/ name label enclosure attached downloads / ) {
                print XML "    <$tag>$attachment[$n]{$tag}</$tag>\n";
            }
            print XML qq{  </attachment>\n};
        }
        for my $cmt ( @comment ) {
            print XML qq{  <comment>\n};
            print XML qq{    <commenter id="">\n};
            for my $tag ( qw/ name homepage ip / ) {
                print XML "      <$tag>$cmt->{commenter}{$tag}</$tag>\n";
            }
            print XML qq{    </commenter>\n};
            for my $tag ( qw/content password secret written isFiltered/ ) {
                print XML "    <$tag>$cmt->{$tag}</$tag>\n";
            }
            print XML qq{  </comment>\n};
        }
        print XML "</post>\n";
    }
}

sub write_gb_xml {
    my @gb = @_;

    my %sth;
    for my $gb ( @gb ) {
        $sth{$gb} = $dbh->prepare( "SELECT * FROM ${prefix}_board_$gb ORDER BY reg_date" );
        $sth{$gb}->execute;
    }

    my %row;
    for ( @gb ) { $row{$_} = '' };
    while (1) {
        for my $gb ( keys %row ) {
            $row{$gb} = $sth{$gb}->fetchrow_hashref if ( !$row{$gb} );
            delete $row{$gb} if ( ! $row{$gb} );
        }
        my $tag;
        my %reg_date;
        for my $gb ( keys %row ) {
            if ( $row{$gb} ) {
                $tag++;
                $reg_date{$gb} = $row{$gb}{reg_date};
            }
        }
        last if ( ! $tag );

        my $gb = (sort { $reg_date{$a} <=> $reg_date{$b} } keys %reg_date)[0];
        my $href = $row{$gb};

        # board encoding (euc-kr to utf-8)
        for my $key ( keys %{$href} ) {
            if ( $key ne 'file_name1' && $key ne 'file_name2' ) {
                 $href->{$key} = xml_encode( $href->{$key} );
            }
            $href->{$key} = encode( "utf-8", decode( "euc-kr", $href->{$key} ) );
        }
        my %data;
        $data{name}     = $href->{name};
        $data{secret}   = $href->{is_secret} ? 1 : 0;
        $data{content}  = "Title: $href->{subject}\n". $href->{memo};
        $data{password} = $href->{password};
        $data{written}  = $href->{reg_date};
        $data{ip}       = $href->{ip};

        my @comment;
        my $sth2 = $dbh->prepare( qq{SELECT * FROM ${prefix}_board_comment_$gb WHERE parent = "$href->{no}" ORDER BY no} );
        $sth2->execute;
        while ( my $cmt = $sth2->fetchrow_hashref ) {
            # comment encoding (euc-kr to utf-8)
            for my $key ( keys %{$cmt} ) {
                $cmt->{$key} = xml_encode( $cmt->{$key} );
                $cmt->{$key} = encode( "utf-8", decode( "euc-kr", $cmt->{$key} ) );
            }
            my $hcmt;
            $hcmt->{commenter}{name}     = $cmt->{name};
            $hcmt->{commenter}{ip}       = $cmt->{ip};
            $hcmt->{password} = $cmt->{password};
            $hcmt->{content}             = $cmt->{memo};
            $hcmt->{secret}              = $data{secret};
            $hcmt->{written}             = $cmt->{reg_date};
            push @comment, $hcmt;
        }

        $row{$gb} = '';

        print XML qq{<guestbook>\n};
        print XML qq{  <comment>\n};
        print XML qq{  <commenter id="">\n};
        for my $tag ( qw/ name ip / ) {
            print XML "    <$tag>$data{$tag}</$tag>\n";
        }
        print XML qq{  </commenter>\n};
        for my $tag ( qw/ content password secret written / ) {
            print XML "    <$tag>$data{$tag}</$tag>\n";
        }
        for my $cmt ( @comment ) {
            print XML qq{    <comment>\n};
            print XML qq{      <commenter id="">\n};
            for my $tag ( qw/ name ip / ) {
                print XML "        <$tag>$cmt->{commenter}{$tag}</$tag>\n";
            }
            print XML qq{      </commenter>\n};
            for my $tag ( qw/ content password secret written / ) {
                print XML "      <$tag>$cmt->{$tag}</$tag>\n";
            }
            print XML qq{  </comment>\n};
        }
        print XML "  </comment>\n";
        print XML "</guestbook>\n";
    }
}

sub xml_encode {
    my $input = shift;
    $input =~ s/&/&amp;/g;
    $input =~ s/\\"/&quot;/g;
    $input =~ s/"/&quot;/g;
    $input =~ s/\\'/&apos;/g;
    $input =~ s/'/&apos;/g;
    $input =~ s/</&lt;/g;
    $input =~ s/>/&gt;/g;
    return $input;
}

sub mime_type {
    my $file = shift;

    my %map = (
         ai  => 'application/postscript',
        aif  => 'audio/x-aiff',
       aifc  => 'audio/x-aiff',
       aiff  => 'audio/x-aiff',
        asc  => 'text/plain',
         au  => 'audio/basic',
        avi  => 'video/x-msvideo',
      bcpio  => 'application/x-bcpio',
        bin  => 'application/octet-stream',
          c  => 'text/plain',
         cc  => 'text/plain',
       ccad  => 'application/clariscad',
        cdf  => 'application/x-netcdf',
      class  => 'application/octet-stream',
       cpio  => 'application/x-cpio',
        cpt  => 'application/mac-compactpro',
        csh  => 'application/x-csh',
        css  => 'text/css',
        dcr  => 'application/x-director',
        dir  => 'application/x-director',
        dms  => 'application/octet-stream',
        doc  => 'application/msword',
        drw  => 'application/drafting',
        dvi  => 'application/x-dvi',
        dwg  => 'application/acad',
        dxf  => 'application/dxf',
        dxr  => 'application/x-director',
        eps  => 'application/postscript',
        etx  => 'text/x-setext',
        exe  => 'application/octet-stream',
         ez  => 'application/andrew-inset',
          f  => 'text/plain',
        f90  => 'text/plain',
        fli  => 'video/x-fli',
        gif  => 'image/gif',
       gtar  => 'application/x-gtar',
         gz  => 'application/x-gzip',
          h  => 'text/plain',
        hdf  => 'application/x-hdf',
         hh  => 'text/plain',
        hqx  => 'application/mac-binhex40',
        htm  => 'text/html',
       html  => 'text/html',
        ice  => 'x-conference/x-cooltalk',
        ief  => 'image/ief',
       iges  => 'model/iges',
        igs  => 'model/iges',
        ips  => 'application/x-ipscript',
        ipx  => 'application/x-ipix',
        jpe  => 'image/jpeg',
       jpeg  => 'image/jpeg',
        jpg  => 'image/jpeg',
         js  => 'application/x-javascript',
        kar  => 'audio/midi',
      latex  => 'application/x-latex',
        lha  => 'application/octet-stream',
        lsp  => 'application/x-lisp',
        lzh  => 'application/octet-stream',
          m  => 'text/plain',
        man  => 'application/x-troff-man',
         me  => 'application/x-troff-me',
       mesh  => 'model/mesh',
        mid  => 'audio/midi',
       midi  => 'audio/midi',
        mif  => 'application/vndmif',
       mime  => 'www/mime',
        mov  => 'video/quicktime',
      movie  => 'video/x-sgi-movie',
        mp2  => 'audio/mpeg',
        mp3  => 'audio/mpeg',
        mpe  => 'video/mpeg',
       mpeg  => 'video/mpeg',
        mpg  => 'video/mpeg',
       mpga  => 'audio/mpeg',
         ms  => 'application/x-troff-ms',
        msh  => 'model/mesh',
         nc  => 'application/x-netcdf',
        oda  => 'application/oda',
        pbm  => 'image/x-portable-bitmap',
        pdb  => 'chemical/x-pdb',
        pdf  => 'application/pdf',
        pgm  => 'image/x-portable-graymap',
        pgn  => 'application/x-chess-pgn',
        png  => 'image/png',
        pnm  => 'image/x-portable-anymap',
        pot  => 'application/mspowerpoint',
        ppm  => 'image/x-portable-pixmap',
        pps  => 'application/mspowerpoint',
        ppt  => 'application/mspowerpoint',
        ppz  => 'application/mspowerpoint',
        pre  => 'application/x-freelance',
        prt  => 'application/pro_eng',
         ps  => 'application/postscript',
         qt  => 'video/quicktime',
         ra  => 'audio/x-realaudio',
        ram  => 'audio/x-pn-realaudio',
        ras  => 'image/cmu-raster',
        rgb  => 'image/x-rgb',
         rm  => 'audio/x-pn-realaudio',
       roff  => 'application/x-troff',
        rpm  => 'audio/x-pn-realaudio-plugin',
        rtf  => 'text/rtf',
        rtx  => 'text/richtext',
        scm  => 'application/x-lotusscreencam',
        set  => 'application/set',
        sgm  => 'text/sgml',
       sgml  => 'text/sgml',
         sh  => 'application/x-sh',
       shar  => 'application/x-shar',
       silo  => 'model/mesh',
        sit  => 'application/x-stuffit',
        skd  => 'application/x-koan',
        skm  => 'application/x-koan',
        skp  => 'application/x-koan',
        skt  => 'application/x-koan',
        smi  => 'application/smil',
       smil  => 'application/smil',
        snd  => 'audio/basic',
        sol  => 'application/solids',
        spl  => 'application/x-futuresplash',
        src  => 'application/x-wais-source',
       step  => 'application/STEP',
        stl  => 'application/SLA',
        stp  => 'application/STEP',
    sv4cpio  => 'application/x-sv4cpio',
     sv4crc  => 'application/x-sv4crc',
        swf  => 'application/x-shockwave-flash',
          t  => 'application/x-troff',
        tar  => 'application/x-tar',
        tcl  => 'application/x-tcl',
        tex  => 'application/x-tex',
       texi  => 'application/x-texinfo',
     texinfo => 'application/x-texinfo',
        tif  => 'image/tiff',
       tiff  => 'image/tiff',
         tr  => 'application/x-troff',
        tsi  => 'audio/TSP-audio',
        tsp  => 'application/dsptype',
        tsv  => 'text/tab-separated-values',
        txt  => 'text/plain',
        unv  => 'application/i-deas',
      ustar  => 'application/x-ustar',
        vcd  => 'application/x-cdlink',
        vda  => 'application/vda',
        viv  => 'video/vndvivo',
       vivo  => 'video/vndvivo',
       vrml  => 'model/vrml',
        wav  => 'audio/x-wav',
        wrl  => 'model/vrml',
        xbm  => 'image/x-xbitmap',
        xlc  => 'application/vndms-excel',
        xll  => 'application/vndms-excel',
        xlm  => 'application/vndms-excel',
        xls  => 'application/vndms-excel',
        xlw  => 'application/vndms-excel',
        xml  => 'text/xml',
        xpm  => 'image/x-xpixmap',
        xwd  => 'image/x-xwindowdump',
        xyz  => 'chemical/x-pdb',
        zip  => 'application/zip',
    );

    $file =~ /\.([^\.]*)$/;
    my $ext = $1;
# convert lower case: edited by chj
    return $map{lc($ext)};
}

